<?php
// $Id: click_heatmap.log.inc,v 1.1.2.13 2008/05/16 06:14:00 boombatower Exp $
/**
 * @file
 * Provides an integration of the ClickHeat utility which creates click heatmaps.
 * 
 * Creates the log review interface.
 *
 * Copyright 2008 by Jimmy Berry ("boombatower", http://drupal.org/user/214218)
 */

/**
 * Menu callback function that creates view logs page.
 */
function click_heatmap_log_view() {
  if (variable_get('click_heatmap_library_installed', FALSE) !== FALSE) {
    drupal_add_css(drupal_get_path('module', 'click_heatmap') .'/style.css');

    if (isset($_GET['clear'])) {
      click_heatmap_log_clear_logs();
    }

    $output = drupal_get_form('click_heatmap_logs');
    if (!isset($_GET['clear']) && $_GET['page'] != '') {
      $output .= click_heatmap_log_generate();
    }

    return $output;
  }
  else {
    return t('Please configure the module first.');
  }
}

/**
 * Creates the log filter form that allows the user to
 * specify what heatmaps they would like to see.
 */
function click_heatmap_logs() {
  $form = array();
  $form['#method'] = 'get';
  $form['display'] = array(
    '#type' => 'fieldset',
    '#title' => t('Display Settings'),
    '#description' => t('Display information that will be used to change the appearance of the heatmaps.'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE, // TRUE
    '#tree' => TRUE
  );
  $form['display']['transparency'] = array(
    '#type' => 'select',
    '#title' => t('Heatmap transparency'),
    '#description' => t('Transparency of the heatmap overlay.'),
    '#default_value' => 30
  );
  $form['display']['width'] = array(
    '#type' => 'select',
    '#title' => t('Display width'),
    '#description' => t('Width of heatmap to display. Heatmap will be scaled to fit.'),
    '#default_value' => 1024
  );
  $form['display']['heatmap'] = array(
    '#type' => 'checkbox',
    '#title' => t('Display heatmap'),
    '#description' => t('Display heatmap instead of clicks.'),
    '#default_value' => TRUE
  );
  $form['display']['update'] = array(
    '#type' => 'submit',
    '#value' => t('Update')
  );

  $form['selection'] = array(
    '#type' => 'fieldset',
    '#title' => t('Selection Information'),
    '#description' => t('Selection information that will be used to narrow the results used to generate heatmaps.'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#tree' => TRUE
  );
  $form['selection']['browser'] = array(
    '#type' => 'select',
    '#title' => t('Browser'),
    '#description' => t('Display clicks from users of a specific browser.'),
    '#default_value' => 'all'
  );
  $form['selection']['date_range'] = array(
    '#type' => 'item',
    '#title' => t('Date range'),
    '#value' => t('Select click data from last 7 days.')
  );
  $form['selection']['update'] = array(
    '#type' => 'submit',
    '#value' => t('Update')
  );

  $form['page'] = array(
    '#type' => 'select',
    '#title' => t('Page'),
    '#description' => t('Display clicks from users on a particular page.'),
    '#attributes' => array('onchange' => 'form.submit()'),
    '#default_value' => $_GET['page']
  );
  $form['update'] = array(
    '#type' => 'submit',
    '#value' => t('Update')
  );
  $form['clear'] = array(
    '#type' => 'submit',
    '#name' => 'clear',
    '#value' => t('Clear Logs')
  );

  $form = array_merge_recursive($form, click_heatmap_log_get_accepted_values()); // Add options to selects.
  click_heatmap_log_clean_values($form); // Clean input values and set defaults.

  if (count($form['page']['#options']) == 0) {
    $form['page']['#disabled'] = TRUE;
  }

  return $form;
}

/**
 * Generate the heatmap and return HTML to display it.
 */
function click_heatmap_log_generate() {
  include_once(drupal_get_path('module', 'click_heatmap') .'/clickheat/classes/Heatmap.class.php');
  include_once(drupal_get_path('module', 'click_heatmap') .'/clickheat/classes/HeatmapFromClicks.class.php');

  $clicks_heatmap = new HeatmapFromClicks();
  $clicks_heatmap->browser = $_GET['selection']['browser'];
  $clicks_heatmap->minScreen = 0;
  $clicks_heatmap->maxScreen = 1024;
  $clicks_heatmap->layout = click_heatmap_log_get_layout($_GET['page']);
  $clicks_heatmap->memory = 64 * 1048576;
  $clicks_heatmap->step = 5;
  $clicks_heatmap->dot = 19;
  $clicks_heatmap->palette = FALSE;
  $clicks_heatmap->heatmap = ($_GET['display']['heatmap'] == '1');
  $clicks_heatmap->path = drupal_get_path('module', 'click_heatmap') .'/heatmaps/';
  $clicks_heatmap->cache = drupal_get_path('module', 'click_heatmap') .'/heatmaps/';
  $clicks_heatmap->file = 'heatmap-%d.png';

  // Add files.
  $days = 7;
  for ($day = 0; $day < $days; $day++) // Last 7 days.
  {
    $date_stamp = time();
    $current_date = date('Y-m-d', mktime(0, 0, 0, date('m', $date_stamp), date('d', $date_stamp) - $day, date('Y', $date_stamp)));
    $clicks_heatmap->addFile(realpath(drupal_get_path('module', 'click_heatmap') .'/clickheat/logs/') .'/'. $_GET['page'] .'/'. $current_date .'.log');
  }

  $result = $clicks_heatmap->generate($_GET['display']['width'], 1024);
  if ($result === false) {
    drupal_set_message(t('Failed to generate heatmap.'));
  }
  else {
    return '<div id="click_heatmap_wrapper"><div id="click_heatmap_overlay"><img src="'. url('admin/logs/click_heatmap/view', NULL, NULL, TRUE) .'" style="opacity: '. (1 - $_GET['display']['transparency'] * 0.01) .';" /></div>
      <iframe id="click_heatmap_webpage" scrolling="no" frameborder="0" src="'. $clicks_heatmap->layout[0] .'" style="width: '. $_GET['display']['width'] .'px; height: 1024px;"></iframe></div>';
  }
}

/**
 * Get the layout for the specified group.
 */
function click_heatmap_log_get_layout($group) {
  $contents = file_get_contents(drupal_get_path('module', 'click_heatmap') .'/clickheat/logs/'. $group .'/url.txt');

  $layout = explode('>', $contents);
  if (count($layout) !== 4) {
    $layout = array('', 0, 0, 0);
  }
  return $layout;
}

/**
 * Get the pages/groups for the current site that contain log data.
 */
function click_heatmap_log_get_pages() {
  $pages = array();

  $site_name = strtolower(str_replace(' ', '.', click_heatmap_site_name()));
  $length = strlen($site_name);
  $dir = opendir(drupal_get_path('module', 'click_heatmap') . '/clickheat/logs/');
  while (($file = readdir($dir)) !== FALSE) {
    if (strpos($file, $site_name) !== FALSE) {
      // Transform directory name into page information.
      // Example: 'drupal.test.area,admin.settings.click_heatmap' => path = admin/settings; page = click_heatmap
      $request = str_replace('.', '/', substr($file, $length + 1));
      $path = (($pos = strrpos($request, '/')) !== FALSE ? substr($request, 0, $pos) : NULL);
      $page = substr($request, ($pos !== FALSE ? $pos + 1 : 0));

      if ($path === NULL) {
        $path = '[ROOT]';
      } 
      $pages[$path][$file] = $page;
    }
  }
  closedir($dir);
  ksort($pages);
  return $pages;
}

/**
 * Menu callback function that outputs generated heatmap image to browser.
 */
function click_heatmap_log_get_image() {
  header('Content-type: image/png');
  readfile(drupal_get_path('module', 'click_heatmap') .'/heatmaps/'.'heatmap-0.png');
}

/**
 * Get the accepted values for the selects displayed on the page. The
 * values are returned in such a way that they can be merged with the
 * form array and fill in the #options attribute.
 */
function click_heatmap_log_get_accepted_values() {
  $values = array();
  $values['display']['transparency']['#options'] = array(10 => '10%', 20 => '20%', 30 => '30%', 40 => '40%', 50 => '50%', 60 => '60%', 70 => '70%', 80 => '80%', 90 => '90%');
  $values['display']['width']['#options'] = $values['display']['width']['#options'] = array(640 => '640px', 800 => '800px', 1024 => '1024px', 1280 => '1280px', 1440 => '1440px', 1600 => '1600px', 1800 => '1800px');
  $values['selection']['browser']['#options'] = array('all' => 'All', 'firefox' => 'Firefox', 'msie' => 'Internet Explorer', 'safari' => 'Safari', 'opera' => 'Opera', 'konqueror' => 'Konqueror', 'unknown' => 'Other/unknown');
  $values['page']['#options'] = click_heatmap_log_get_pages();
  return $values;
}

/**
 * Clean the incomming filter values and set the default value for
 * form inputs.
 */
function click_heatmap_log_clean_values(&$form) {
  // Get accepted values.
  $accepted = click_heatmap_log_get_accepted_values();

  // Clean display settings and set form default values to correct value.
  if (!array_key_exists($_GET['display']['transparency'], $accepted['display']['transparency']['#options'])) {
    $_GET['display']['transparency'] = $form['display']['transparency']['#default_value'];
  }
  else {
    $form['display']['transparency']['#default_value'] = $_GET['display']['transparency'];
  }

  if (!array_key_exists($_GET['display']['width'], $accepted['display']['width']['#options'])) {
    $_GET['display']['width'] = $form['display']['width']['#default_value'];
  }
  else {
    $form['display']['width']['#default_value'] = $_GET['display']['width'];
  }

  if ($_GET['form_id'] == 'click_heatmap_logs') {
    if ($_GET['display']['heatmap'] != TRUE && $_GET['display']['heatmap'] != '') {
      $_GET['display']['heatmap'] = $form['display']['heatmap']['#default_value'];
    }
    else {
      $form['display']['heatmap']['#default_value'] = $_GET['display']['heatmap'];
    }
  }
  else {
    $_GET['display']['heatmap'] = $form['display']['heatmap']['#default_value'];
  }

  if (!array_key_exists($_GET['selection']['browser'], $accepted['selection']['browser']['#options'])) {
    $_GET['selection']['browser'] = $form['selection']['browser']['#default_value'];
  }
  else {
    $form['selection']['browser']['#default_value'] = $_GET['selection']['browser'];
  }

  if (!array_key_exists($_GET['page'], $accepted['page']['#options'])) {
    $_GET['page'] = $form['page']['#default_value'];
  }
  else {
    $form['page']['#default_value'] = $_GET['page'];
  }
}

/**
 * Clear the log cache.
 */
function click_heatmap_log_clear_logs() {
  $logs_directory = opendir(drupal_get_path('module', 'click_heatmap') . '/clickheat/logs/');

  $error = FALSE;
  while (($log = readdir($logs_directory)) !== FALSE) {
    if (click_heatmap_log_valid_file($log)) { // Files to ignore.
      // Get log files in directory.
      $directory = drupal_get_path('module', 'click_heatmap') .'/clickheat/logs/'. $log;
      $log_directory = opendir($directory);
      while (($file = readdir($log_directory)) !== FALSE) { // Remove files in directory first.
        if (click_heatmap_log_valid_file($file)) {
          if (!unlink($directory .'/'. $file)) {
            drupal_set_message(t('Failed to delete log: %file.', array('%file' => $file)), 'error');
            $error = TRUE;
          }
        }
      }
      closedir($log_directory);

      if (!rmdir($directory)) { // Remove directory.
        drupal_set_message(t('Failed to delete log directory: %directory.', array('%directory' => $log)), 'error');
        $error = TRUE;
      }
    }
  }
  closedir($logs_directory);

  if (!$error) {
    drupal_set_message(t('Logs cleared.'));
  }
}

/**
 * Check if the specified file is valid to remove.
 */
function click_heatmap_log_valid_file($file) {
  return ($file != '.' && $file != '..' && $file != '.htaccess');
}
